var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "./config-manager", "./file-utils", "path", "./npm-manager", "./environment-params"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var config_manager_1 = require("./config-manager");
    var FileUtils = require("./file-utils");
    var path = require("path");
    var npm_manager_1 = require("./npm-manager");
    var environment_params_1 = require("./environment-params");
    /*
        This class update babel configuration to sealights requirements
     */
    var BabelConfigManager = /** @class */ (function (_super) {
        __extends(BabelConfigManager, _super);
        function BabelConfigManager(consoleLogger, environmentParams, nodeEnvValues, logger) {
            var _this = _super.call(this, consoleLogger, config_manager_1.Packages.BABEL, environmentParams, logger) || this;
            _this.requiredDependencies = {
                "babel-plugin-istanbul": "^3.1.2",
                "babel-preset-env": "^1.1.8"
            };
            _this.defaultConfig = {
                presets: [
                    "env",
                ],
                plugins: [
                    "transform-object-rest-spread",
                ],
                env: {
                    test: {
                        plugins: ["istanbul"],
                    },
                },
            };
            _this.nodeEnvValues = nodeEnvValues;
            return _this;
        }
        BabelConfigManager.prototype.update = function (config) {
            this.saveOriginalConfig(config);
            this.updatePresets(config);
            this.updatePlugins(config);
            this.updateEnv(config);
            return config;
        };
        ;
        BabelConfigManager.prototype.writeDefault = function () {
            var configToSave = Object.assign({}, this.defaultConfig);
            configToSave.env = {};
            this.updateEnv(configToSave);
            return this.writeToConfigFile(configToSave);
        };
        BabelConfigManager.prototype.updateEnv = function (config) {
            var _this = this;
            if (!config.env) {
                config.env = {};
            }
            this.nodeEnvValues.forEach(function (envName) {
                if (!config.env[envName]) {
                    config.env[envName] = _this.defaultConfig.env.test;
                    return;
                }
                if (!config.env[envName].plugins) {
                    config.env[envName].plugins = _this.defaultConfig.env.test.plugins;
                    return;
                }
                var plugins = config.env[envName].plugins;
                if (!plugins.includes(_this.defaultConfig.env.test.plugins[0])) {
                    plugins.push(_this.defaultConfig.env.test.plugins[0]);
                }
            });
        };
        BabelConfigManager.prototype.updatePlugins = function (config) {
            if (!config.plugins) {
                config.plugins = this.defaultConfig.plugins;
                return;
            }
            if (config.plugins.indexOf(this.defaultConfig.plugins[0]) == -1) {
                config.plugins.push(this.defaultConfig.plugins[0]);
            }
        };
        BabelConfigManager.prototype.updatePresets = function (config) {
            if (!config.presets) {
                config.presets = this.defaultConfig.presets;
                return;
            }
            if (config.presets.indexOf(this.defaultConfig.presets[0]) == -1) {
                config.presets.push(this.defaultConfig.presets[0]);
            }
        };
        BabelConfigManager.prototype.updateDependencies = function () {
            var environmentParams = new environment_params_1.EnvironmentParams();
            var packagePath = path.resolve(environmentParams.getBasePath(), "package.json");
            try {
                var content = FileUtils.readJsonFileSafe(packagePath, this.consoleLogger);
                var dependencies = content.dependencies || {};
                var fullDependencies = Object.assign(dependencies, this.requiredDependencies);
                content.dependencies = fullDependencies;
                FileUtils.writeJsonFileSafe(packagePath, content, this.consoleLogger);
                this.installDependencies();
            }
            catch (err) {
                this.consoleLogger.printError(err);
                throw err;
            }
        };
        BabelConfigManager.prototype.installDependencies = function () {
            try {
                var packagesToInstall = Object.keys(this.requiredDependencies);
                var npmManager = new npm_manager_1.NpmManager(this.logger);
                var environmentParams = new environment_params_1.EnvironmentParams();
                npmManager.installPackages(packagesToInstall, environmentParams.getBasePath());
            }
            catch (err) {
                throw err;
            }
        };
        BabelConfigManager.prototype.getFileName = function () {
            return ".babelrc";
        };
        BabelConfigManager.prototype.getConfigKey = function () {
            return "babel";
        };
        BabelConfigManager.prototype.tryUpdate = function () {
            try {
                var originalConfig = this.read();
                var isSecceed = void 0;
                if (!originalConfig) {
                    this.consoleLogger.println("Can't find configuration for babel  in project. default configuration will be saved to .babelrc");
                    isSecceed = this.writeDefault();
                }
                else {
                    var updatedConfig = this.update(originalConfig);
                    isSecceed = this.write(updatedConfig);
                }
                if (isSecceed)
                    this.updateDependencies();
                return isSecceed;
            }
            catch (err) {
                this.consoleLogger.printError("Failed to update babel configuration - skip this phase");
                return false;
            }
        };
        ;
        return BabelConfigManager;
    }(config_manager_1.ConfigManager));
    exports.BabelConfigManager = BabelConfigManager;
});
//# sourceMappingURL=babel-config-manager.js.map