(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "fs", "sl-js-infra", "./file-utils", "./logger"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var fs = require("fs");
    var sl_js_infra_1 = require("sl-js-infra");
    var file_utils_1 = require("./file-utils");
    var logger_1 = require("./logger");
    var ValidationUtils = sl_js_infra_1.SlJsInfra.Utils.ValidationUtils;
    /*
        This class restores folder that backed up by sealights
        it's delete the given folder, and replace it by same folder with '.slback' extension.
     */
    var FolderRestore = /** @class */ (function () {
        function FolderRestore(folderToRestore) {
            ValidationUtils.verifyNotNullOrEmpty(folderToRestore, "folderToRestore");
            this.folderToRestore = folderToRestore;
            this.sourceFolder = folderToRestore + FolderRestore.BACKED_FOLDER_EXTENSION;
            this.logger = logger_1.LoggerFactory.getCreateApplicationLogger();
        }
        FolderRestore.prototype.restore = function () {
            if (!this.isSourceFolderExist()) {
                this.logger.warn("Could not find source folder " + this.sourceFolder + " skipping restore process.");
                throw new Error("Folder " + this.sourceFolder + " dose not exist");
            }
            file_utils_1.deleteFolder(this.folderToRestore);
            file_utils_1.renameFolder(this.sourceFolder, this.folderToRestore);
        };
        FolderRestore.prototype.isSourceFolderExist = function () {
            return fs.existsSync(this.sourceFolder);
        };
        FolderRestore.BACKED_FOLDER_EXTENSION = ".slback";
        return FolderRestore;
    }());
    exports.FolderRestore = FolderRestore;
});
//# sourceMappingURL=folder-restore.js.map