(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "fs", "path", "../../environment-params"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var fs = require("fs");
    var path = require("path");
    var environment_params_1 = require("../../environment-params");
    var BrowserAgentInstrumentor = /** @class */ (function () {
        function BrowserAgentInstrumentor(config, userMessageHelper) {
            this.config = config;
            this.userMessageHelper = userMessageHelper;
            if (config == null)
                throw ("'config' is required");
            if (userMessageHelper == null)
                throw ("'userMessageHelper' is required");
        }
        BrowserAgentInstrumentor.prototype.instrument = function (sealightsConfig) {
            var agentDirPath = BrowserAgentInstrumentor.getAgentDirectory();
            var agentPath = agentDirPath + "/browser-agent-all.js";
            //Create the data needed for the final agent file.
            var agentFileData = fs.readFileSync(agentPath, "utf-8");
            var preamble = this.createPreamble(sealightsConfig);
            //Save the instrumented agent
            var instrumentedAgentPath = BrowserAgentInstrumentor.getInstrumentedAgentPath();
            fs.writeFileSync(instrumentedAgentPath, preamble + agentFileData, "utf-8");
        };
        BrowserAgentInstrumentor.getAgentDirectory = function () {
            var agentDirPath = path.dirname(require.resolve('sl-browser-agent'));
            return agentDirPath;
        };
        BrowserAgentInstrumentor.getInstrumentedAgentPath = function () {
            var agentDirPath = BrowserAgentInstrumentor.getAgentDirectory();
            var instrumentedAgentPath = agentDirPath + "/inst-browser-agent-all.js";
            instrumentedAgentPath = BrowserAgentInstrumentor.adjustPathSlashes(instrumentedAgentPath);
            return instrumentedAgentPath;
        };
        BrowserAgentInstrumentor.getSourceMapPackagePath = function () {
            var sourceMapPath = path.dirname(require.resolve('source-map'));
            sourceMapPath = sourceMapPath + "/dist/source-map.js";
            sourceMapPath = BrowserAgentInstrumentor.adjustPathSlashes(sourceMapPath);
            return sourceMapPath;
        };
        BrowserAgentInstrumentor.prototype.resolveWorkspacepath = function () {
            var workspacepath = this.config.workspacepath;
            if (!workspacepath) {
                var environmentParams = new environment_params_1.EnvironmentParams();
                this.userMessageHelper.println("'workspacepath' not configured for the browser agent. Using current working directory: " + environmentParams.getBasePath());
                workspacepath = environmentParams.getBasePath();
            }
            if (!fs.existsSync(workspacepath)) {
                this.userMessageHelper.println("'workspacepath' doesn't exist. Please verify that the path is correct.");
            }
            // '/' is important for relative path resolving on the browser agent (in order for the footprints of the browser agent to be relative to the workspace path)
            workspacepath = path.resolve(workspacepath);
            workspacepath = BrowserAgentInstrumentor.adjustPathSlashes(workspacepath);
            if (workspacepath.substr(-1) !== "/") {
                workspacepath += "/";
            }
            return workspacepath;
        };
        BrowserAgentInstrumentor.adjustPathSlashes = function (path) {
            path = (path || "").replace(/\\/g, '/');
            return path;
        };
        BrowserAgentInstrumentor.prototype.createPreamble = function (sealightsConfig) {
            var workspacepath = this.resolveWorkspacepath();
            var preamble = "if (!window.$Sealights) window.$Sealights = " + JSON.stringify({
                customerId: this.config.customerId || (sealightsConfig.customerId && sealightsConfig.customerId.value),
                appName: this.config.appName || (sealightsConfig.appName && sealightsConfig.appName.value),
                buildName: this.config.build || (sealightsConfig.build && sealightsConfig.build.value),
                branchName: this.config.branch || (sealightsConfig.branch && sealightsConfig.branch.value),
                server: this.config.server || (sealightsConfig.server && sealightsConfig.server.value),
                token: this.config.token || (sealightsConfig.token && sealightsConfig.token.value),
                buildSessionId: this.config.buildSessionId || (this.config.buildsessionid && sealightsConfig.buildsessionid.value),
                labId: this.config.labId || (sealightsConfig.labId && sealightsConfig.labId.value),
                testStage: this.config.testStage || (sealightsConfig.testStage && sealightsConfig.testStage.value),
                enabled: true,
                workspacepath: workspacepath,
                maxItemsInQueue: 500,
                registerShutdownHook: true,
                interval: 10,
                resolveWithoutHash: this.config.resolveWithoutHash || false
            }, null, '  ') + ";";
            return preamble;
        };
        return BrowserAgentInstrumentor;
    }());
    exports.BrowserAgentInstrumentor = BrowserAgentInstrumentor;
});
//# sourceMappingURL=browser-agent-instrumentor.js.map