(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var sealights = require('sl-node2');
    var SealightsReporter = /** @class */ (function () {
        function SealightsReporter() {
            this.executionId = null;
            this.slProcess = null;
        }
        SealightsReporter.prototype.reportRunnerStarting = function (runner) {
            var that = this;
            this.log("[SeaLights for Jasmine] Agent version " + sealights.getVersion() + " is initializing");
            try {
                that.slProcess = sealights.init({});
                that.executionId = that.slProcess.createTestSuiteId();
                that.slProcess.pushEvent({ type: 'executionIdStarted', executionId: this.executionId, framework: 'jasmine', frameworkVersion: "1.x" });
                var __initTestId = that.slProcess.createTestId(this.executionId, '__init');
                that.slProcess.setCurrentTestIdentifier(__initTestId, true);
                this.log('[SeaLights for Jasmine] Initialized');
            }
            catch (e) {
                this.log('[SeaLights for Jasmine] Initialization error: ' + (e.stack || e));
            }
            ;
        };
        SealightsReporter.prototype.createTestEvent = function (spec, eventType) {
            var suitePath = SealightsReporter.getSuitePath(spec);
            var testName = suitePath.concat(spec.description).join(' '); //spec.description;
            return { type: eventType, testName: testName, executionId: this.executionId, testSuitePath: suitePath, result: null };
        };
        SealightsReporter.prototype.reportSpecStarting = function (spec) {
            var evt = this.createTestEvent(spec, "testStart");
            this.slProcess.pushEvent(evt);
            var newTestId = this.slProcess.createTestId(this.executionId, evt.testName);
            this.slProcess.setCurrentTestIdentifier(newTestId);
        };
        SealightsReporter.prototype.reportSpecResults = function (spec) {
            var evt = this.createTestEvent(spec, "testEnd");
            var results = spec.results();
            if (results.skipped) {
                evt.result = "skipped";
            }
            else if (results.passed()) {
                evt.result = "passed";
            }
            else {
                evt.result = "failed";
            }
            this.slProcess.pushEvent(evt);
        };
        SealightsReporter.prototype.reportRunnerResults = function (runner) {
            var _this = this;
            this.slProcess.setCurrentTestIdentifier(null);
            this.slProcess.pushEvent({ type: 'executionIdEnded', executionId: this.executionId });
            this.slProcess.stop(function () {
                _this.log("[SeaLights for Jasmine] Test listener is shutting down.");
            });
        };
        SealightsReporter.prototype.log = function (str) {
            var console = jasmine.getGlobal().console;
            if (console && console.log) {
                console.log(str);
            }
        };
        SealightsReporter.getSuitePath = function (spec) {
            var parts = [];
            var suite = spec.suite;
            while (suite) {
                parts.unshift(suite.description);
                suite = suite.parentSuite;
            }
            return parts;
        };
        return SealightsReporter;
    }());
    exports.SealightsReporter = SealightsReporter;
    var jasmine = global && global["jasmine"];
    typeof (jasmine) !== 'undefined' && jasmine.getEnv() && jasmine.getEnv().addReporter(new SealightsReporter());
});
//# sourceMappingURL=jasmine-1.x.js.map