(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "../../configuration-manager", "../commons/browser-agent-instrumentor", "../../user-message-helper"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var configuration_manager_1 = require("../../configuration-manager");
    var browser_agent_instrumentor_1 = require("../commons/browser-agent-instrumentor");
    var user_message_helper_1 = require("../../user-message-helper");
    var SlPreProcessor = /** @class */ (function () {
        function SlPreProcessor(config, logger) {
            this.config = config;
            this.logger = logger;
        }
        SlPreProcessor.prototype.createPreProcessingFunction = function (utils) {
            var context = this;
            this.utils = utils;
            var log = context.logger.create('sl-karma-preprocessor');
            log.info("SlPreProcessor has started.");
            return function (content, file, done) {
                log.debug("Pre processing function has started.");
                if (SlPreProcessor.instrumented) {
                    log.info("No need to instrument the browser agent as the we already have an instrumented version.");
                    return done(content);
                }
                var configurationManager = new configuration_manager_1.ConfigurationManager();
                var convertedConfig = configurationManager.convertConfigParams(context.config);
                context.utils.configTestFramework(convertedConfig, function (err) {
                    try {
                        if (err) {
                            log.warn("Failed configuring the agent. " + err);
                            done(null, content);
                        }
                        else {
                            var userMessageHelper = new user_message_helper_1.UserMessageHelper("Sealights For Karma", false);
                            // TODO: Remove code if ci pass
                            /*
                            **********************************************************************
                            *  Commented out since karma integration is handled on Browser Agent *
                            **********************************************************************
                            var slAgent = Agent.getAgent();
                            if (!slAgent) {
                                log.info("Creating Sealights Agent.")
                                Agent.initAgent(log, userMessageHelper);
                            }
                            else
                            {
                                log.info("Sealights Agent was already initialized.")
                            }
                            */
                            var sealightsConfig = configurationManager.getSealightsConfiguration();
                            var browserAgentInstrumentor = new browser_agent_instrumentor_1.BrowserAgentInstrumentor(context.config, userMessageHelper);
                            browserAgentInstrumentor.instrument(sealightsConfig);
                            SlPreProcessor.instrumented = true;
                            log.debug("Created an instrumented version of the Browser Agent. sealightsConfig: ", sealightsConfig);
                            done(null, content);
                        }
                    }
                    catch (err) {
                        log.warn("Failed instrumenting the agent. " + err);
                        done(null, content);
                    }
                });
            };
        };
        SlPreProcessor.instrumented = false;
        //This field is being used by Karma.
        SlPreProcessor.$inject = ['config.sealights', 'logger'];
        return SlPreProcessor;
    }());
    exports.SlPreProcessor = SlPreProcessor;
});
//# sourceMappingURL=preprocessor.js.map