(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "fs", "../../utils", "../commons/browser-agent-instrumentor", "../../environment-params"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var fs = require("fs");
    var utils_1 = require("../../utils");
    var browser_agent_instrumentor_1 = require("../commons/browser-agent-instrumentor");
    var environment_params_1 = require("../../environment-params");
    var MochaPhantomJs = /** @class */ (function () {
        function MochaPhantomJs(browserAgentInstrumentor, userMessageHelper, configurationManager) {
            this.browserAgentInstrumentor = browserAgentInstrumentor;
            this.userMessageHelper = userMessageHelper;
            this.configurationManager = configurationManager;
            if (browserAgentInstrumentor == null) {
                throw new Error("'browserAgentInstrumentor' cannot be null");
            }
            if (userMessageHelper == null) {
                throw new Error("'userMessageHelper' cannot be null");
            }
            if (configurationManager == null) {
                throw new Error("'configurationManager' cannot be null");
            }
        }
        MochaPhantomJs.prototype.integrate = function (convertedConfig, callback) {
            var _this = this;
            if (convertedConfig == null) {
                throw new Error("'convertedConfig' cannot be null");
            }
            if (callback == null) {
                throw new Error("'callback' cannot be null");
            }
            var utils = new utils_1.Utils();
            utils.configTestFramework(convertedConfig, function (err) {
                try {
                    if (err) {
                        _this.userMessageHelper.printError("Failed while configuring the agent. " + err);
                        return callback(err);
                    }
                    var sealightsConfig = _this.configurationManager.getSealightsConfiguration();
                    _this.browserAgentInstrumentor.instrument(sealightsConfig);
                    _this.createSlHooksFile();
                    return callback(null);
                }
                catch (err) {
                    _this.userMessageHelper.printError("Failed instrumenting the agent. " + err.message + "\n" + err.stack);
                    callback(err);
                }
            });
        };
        MochaPhantomJs.prototype.createSlHooksFile = function () {
            var instrumentedAgentPath = browser_agent_instrumentor_1.BrowserAgentInstrumentor.getInstrumentedAgentPath();
            var sourceMapPath = browser_agent_instrumentor_1.BrowserAgentInstrumentor.getSourceMapPackagePath();
            var userMessageHelperCode = fs.readFileSync(require.resolve('../commons/user-message-helper'), { encoding: "utf8" });
            var isDebugEnabled = (process.env["NODE_DEBUG"] && process.env["NODE_DEBUG"].toLowerCase() === "sl");
            var environmentParams = new environment_params_1.EnvironmentParams();
            var content = fs.readFileSync(MochaPhantomJs.SL_HOOKS_TEMPLATE_PATH, { encoding: "utf8" });
            content = content.replace("@@ADDITONAL_SCRIPTS@@", userMessageHelperCode);
            content = content.replace("@@INSTRUMENTED_AGENT_PATH@@", instrumentedAgentPath);
            content = content.replace("@@SOURCE_MAP_PATH@@", sourceMapPath);
            content = content.replace("@@IS_DEBUG_ENABLED@@", isDebugEnabled.toString());
            var slHooksPath = environmentParams.getBasePath() + "/sl-hooks.js";
            fs.writeFileSync(slHooksPath, content, "utf8");
        };
        MochaPhantomJs.SL_HOOKS_TEMPLATE_PATH = __dirname + "/sl-hooks.js.template";
        return MochaPhantomJs;
    }());
    exports.MochaPhantomJs = MochaPhantomJs;
});
//# sourceMappingURL=mocha-phantomjs.js.map