/**
 * Usage in webpack.conf
 * var SlWebpackPlugin = require("slnodejs").SlWebpackPlugin;
 *
 * plugins: [
      new SlWebpackPlugin({"workspacepath": "dist", "scm": "git", "tokenFile": "./tk"},
                          {"branch": "origin/master", "build": "1", "appName": "sl-cloud-ui", "tokenFile": "./tk"})
   ]
 * 1) devtool: 'source-map' must be turned on
 * https://webpack.js.org/configuration/devtool/#devtool
 *
 * 2) output.sourceMapFilename must be of the form "[file].map" which is the default.
 * https://webpack.js.org/configuration/output/#output-sourcemapfilename
 */
(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "../../sl-cia-proxy"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var sl_cia_proxy_1 = require("../../sl-cia-proxy");
    var utils = require("../../utils.js");
    //TODO: Seems like this could be removed
    //var cli = require.resolve("../../cli.js"); 
    var SlWebpackPlugin = /** @class */ (function () {
        function SlWebpackPlugin(scanOptions, configOptions) {
            this.scanOptions = scanOptions;
            this.configOptions = configOptions;
        }
        SlWebpackPlugin.prototype.apply = function (compiler) {
            var context = this;
            compiler.plugin("after-emit", function (compilation, callback) {
                if (!compilation || !compilation.options || compilation.options.devtool !== "source-map") {
                    console.warn("[SEALIGHTS] webpack devtool option must be set to 'source-map'");
                }
                var ciaProxy = new sl_cia_proxy_1.SlCiaProxy();
                if (context.configOptions) {
                    context.convertConfigParams(context.configOptions);
                    ciaProxy.config(context.configOptions, function (err) {
                        if (err) {
                            console.warn("Failed Build Config. Skipping Build Scan. " + err);
                            callback(utils.handleError(context.configOptions, err));
                        }
                        else {
                            context.convertBuildParams(context.scanOptions);
                            ciaProxy.build(context.scanOptions, function (err) {
                                if (err) {
                                    console.warn("Failed Scanning Build. " + err);
                                    callback(utils.handleError(context.scanOptions, err));
                                }
                                else {
                                    callback(null);
                                }
                            });
                        }
                    });
                }
                else {
                    context.convertBuildParams(context.scanOptions);
                    ciaProxy.build(context.scanOptions, function (err) {
                        if (err) {
                            console.warn("Failed Scanning Build. " + err);
                            callback(utils.handleError(context.scanOptions, err));
                        }
                        else {
                            callback(null);
                        }
                    });
                }
            });
        };
        ;
        SlWebpackPlugin.prototype.convertConfigParams = function (configOptions) {
            configOptions.tokenfile = configOptions.tokenfile || configOptions.tokenFile;
            configOptions.appname = configOptions.appname || configOptions.appName;
            configOptions.buildsessionid = configOptions.buildsessionid || configOptions.buildSessionId;
            configOptions.failbuild = configOptions.failbuild || configOptions.failBuild || false;
        };
        SlWebpackPlugin.prototype.convertBuildParams = function (scanOptions) {
            scanOptions.tokenfile = scanOptions.tokenfile || scanOptions.tokenFile;
            scanOptions.appname = scanOptions.appname || scanOptions.appName;
            scanOptions.buildsessionid = scanOptions.buildsessionid || scanOptions.buildSessionId;
            scanOptions.buildsessionidfile = scanOptions.buildsessionidfile || scanOptions.buildSessionIdFile;
            scanOptions.usebranchcoverage = scanOptions.usebranchcoverage || scanOptions.useBranchCoverage || true;
            scanOptions.failbuild = scanOptions.failbuild || scanOptions.failBuild || false;
            scanOptions.recursive = scanOptions.recursive || true;
            scanOptions.instrumentForBrowsers = scanOptions.instrumentforbrowsers || scanOptions.instrumentForBrowsers || false;
            scanOptions.es6Modules = scanOptions.es6modules || scanOptions.es6Modules || false;
            scanOptions.useNewBuildScannerProtocol = scanOptions.resolvewithouthash || scanOptions.resolveWithoutHash || false;
            scanOptions.instrumentationOnly = scanOptions.instrumentationonly || scanOptions.instrumentationOnly;
            scanOptions.sendContributors = scanOptions.sendcontributors || scanOptions.sendContributors;
            scanOptions.delayShutdownInSeconds = scanOptions.delayshutdowninseconds || scanOptions.delayShutdownInSeconds;
            scanOptions.downloadAgent = scanOptions.downloadagent || scanOptions.downloadAgent || true;
            scanOptions.uniqueModuleId = scanOptions.uniquemoduleid || scanOptions.uniqueModuleId;
        };
        return SlWebpackPlugin;
    }());
    exports.SlWebpackPlugin = SlWebpackPlugin;
    module.exports.SlWebpackPlugin = SlWebpackPlugin;
});
//# sourceMappingURL=plugin.js.map