(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "child_process"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var child_Process = require("child_process");
    /*
        This class handles all npm operation that needed.
     */
    var NpmManager = /** @class */ (function () {
        function NpmManager(logger) {
            this.logger = logger;
        }
        NpmManager.prototype.removePackages = function (packages, basePath) {
            var npmCommand = this.getNpmCommand();
            var result = child_Process.spawnSync(npmCommand, ["uninstall"].concat(packages), { cwd: basePath });
            if (result.error || result.status !== 0) {
                throw new Error("Failed to remove packages '" + packages.join(", ") + "'");
            }
            this.logger.info("No error while remove packages: '" + packages.join(", ") + "'");
        };
        NpmManager.prototype.installSpecificVersion = function (packageToInstall, version, basePath) {
            if (!packageToInstall) {
                throw new Error("Package name must be given.");
            }
            if (!version) {
                throw new Error("Version name must be given.");
            }
            if (!basePath) {
                throw new Error("Base path must be given.");
            }
            var packageWithVersion = packageToInstall + "@" + version;
            var npmCommand = this.getNpmCommand();
            var result = child_Process.spawnSync(npmCommand, ["i", packageWithVersion], { cwd: basePath });
            if (result.error || result.status !== 0) {
                throw new Error("Failed to install version '" + version + "' for: '" + packageToInstall + "'");
            }
            this.logger.info("No errors while installing version '" + version + "' for: '" + packageToInstall + "'");
        };
        NpmManager.prototype.installFromFile = function (filename, basePath) {
            if (!filename) {
                throw new Error("File name must be given.");
            }
            if (!basePath) {
                throw new Error("Base path must be given.");
            }
            var npmCommand = this.getNpmCommand();
            var result = child_Process.spawnSync(npmCommand, ["i", filename], { cwd: basePath });
            if (result.error || result.status !== 0) {
                var msg = result.error || "status code is not 0";
                throw new Error("Failed to install file '" + filename + "' Error: '" + msg + "'");
            }
            this.logger.info("No errors while installing version '" + filename + "'");
        };
        NpmManager.prototype.installPackages = function (packagesToInstall, basePath) {
            if (!packagesToInstall) {
                throw new Error("Package name must be given.");
            }
            if (!basePath) {
                throw new Error("Base path must be given.");
            }
            var command = this.getNpmCommand();
            var result = child_Process.spawnSync(command, ["i"].concat(packagesToInstall), { cwd: basePath });
            if (result.error || result.status !== 0) {
                throw new Error("Failed to install packages: '" + packagesToInstall.join(", ") + "'");
            }
            this.logger.info("No error while installing packages: '" + packagesToInstall.join(", ") + "'");
        };
        NpmManager.prototype.getNpmCommand = function () {
            var isWin = /^win/.test(process.platform);
            return isWin ? "npm.cmd" : "npm";
        };
        return NpmManager;
    }());
    exports.NpmManager = NpmManager;
});
//# sourceMappingURL=npm-manager.js.map