(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "./utils", "fs"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var utils_1 = require("./utils");
    var fs = require("fs");
    var CIA = require("sl-cia").CIA;
    /**
     * Left for backward compatibility, webpack plugin also uses this class.
     * @param cmd
     * @param callback
     */
    var SlCiaProxy = /** @class */ (function () {
        function SlCiaProxy() {
            this.utils = new utils_1.Utils();
        }
        SlCiaProxy.prototype.config = function (cmd, callback) {
            var _this = this;
            try {
                this.utils.validateRequiredParams(cmd, ["appname", "branch", "build"], [["token", "tokenfile"]]);
                this.utils.parseToken(cmd);
                this.utils.validateParams(cmd);
                this.utils.createBuildSessionId(cmd, function (err, buildSessionId) {
                    if (err) {
                        console.log("Failed Creating Build Session Id. " + err);
                        callback(_this.utils.handleError(cmd, err));
                    }
                    else {
                        fs.writeFileSync("buildSessionId", buildSessionId);
                        console.log("Build Session Id Created Successfully In buildSessionId File. Build Session Id: " + buildSessionId);
                        callback(null);
                    }
                });
            }
            catch (err) {
                callback(this.utils.handleError(cmd, err));
            }
        };
        SlCiaProxy.prototype.build = function (cmd, callback) {
            var _this = this;
            try {
                this.utils.validateRequiredParams(cmd, ["workspacepath", "scm"], [
                    ["token", "tokenfile"],
                    ["buildsessionid", "buildsessionidfile"]
                ]);
                var buildSessionId = this.utils.getBuildSession(cmd);
                this.utils.parseAndGetParams(cmd, buildSessionId, function (err) {
                    if (err) {
                        console.log("Failed Running Build Scanner. " + err);
                        callback(_this.utils.handleError(cmd, err));
                    }
                    else {
                        _this.runCia(cmd, callback);
                    }
                });
            }
            catch (err) {
                callback(this.utils.handleError(cmd, err));
            }
        };
        SlCiaProxy.prototype.runCia = function (cmd, callback) {
            new CIA().run(cmd, callback);
        };
        return SlCiaProxy;
    }());
    exports.SlCiaProxy = SlCiaProxy;
});
//# sourceMappingURL=sl-cia-proxy.js.map