(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "child_process", "fs", "path", "jwt-decode", "semver", "./default-file-utils", "./logger", "sl-js-infra", "./agent-instance-data"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var childProcess = require("child_process");
    var fs = require("fs");
    var path = require("path");
    var jwtDecode = require("jwt-decode");
    var semver = require("semver");
    var DefaultFileUtils = require("./default-file-utils");
    var logger_1 = require("./logger");
    var sl_js_infra_1 = require("sl-js-infra");
    var agent_instance_data_1 = require("./agent-instance-data");
    var SEALIGHTS = "./sealights.json";
    var JWTPrefix = "eyJhbGciOiJSUzUxMiIsInR5cCI6IkpXVCJ9.";
    var defualtTokenFile = "sltoken.txt";
    var tokenFileKey = "tokenFile";
    var defualtBuildSessionIdFile = "buildSessionId";
    var buildSessionIdFileKey = "buildsessionidfile";
    var slEnvVars = sl_js_infra_1.SlJsInfra.SlEnvVars;
    /*
        This array contains the v8 options that can be passed to sealights child processes.
        V8 options allow words to be separated by both dashes (-) or underscores (_).
        For example, --max-old-space-size is equivalent to --max_old_space_size.
        We save all options with underscores (_). method 'getSpawnNodeArgs' should handle the alignment.
     */
    var allowedV8Options = ["max_old_space_size"];
    function getDependenciesPaths() {
        var deps = {
            slCiaPath: "sl-cia",
            slNodePath: "sl-node",
            slNode2Path: "sl-node2",
            slNodeCoverPath: "sl-node-cover",
            slNodeMochaPath: "sl-node-mocha",
            slNodeMocha2Path: "sl-node-mocha2"
        };
        if (semver.lt(process.versions.node, "5.0.0")) {
            deps.slCiaPath = "slnodejs/node_modules/sl-cia";
            deps.slNodePath = "slnodejs/node_modules/sl-node";
            deps.slNode2Path = "slnodejs/node_modules/sl-node2";
            deps.slNodeCoverPath = "slnodejs/node_modules/sl-node/node_modules/sl-node-cover";
            deps.slNodeMochaPath = "slnodejs/node_modules/sl-node-mocha";
            deps.slNodeMocha2Path = "slnodejs/node_modules/sl-node-mocha2";
        }
        return deps;
    }
    var Utils = /** @class */ (function () {
        function Utils() {
            this.logger = logger_1.LoggerFactory.getCreateApplicationLogger();
        }
        Utils.prototype.deps = function () {
            return getDependenciesPaths();
        };
        Utils.prototype.booleanOption = function (option) {
            return (option && option.toLowerCase() == "true");
        };
        Utils.prototype.csvToArray = function (raw) {
            var trimmedValues = [];
            raw.split(',').forEach(function (val) {
                if (val)
                    trimmedValues.push(val.trim());
            });
            return trimmedValues;
        };
        Utils.prototype.createParamsFromCommand = function (argsMap, cmd) {
            var params = [];
            for (var argName in argsMap) {
                if (cmd.hasOwnProperty(argName) && cmd[argName] !== undefined) {
                    params.push("--" + argsMap[argName]);
                    params.push(cmd[argName]);
                }
            }
            return params;
        };
        Utils.prototype.parseToken = function (cmd) {
            var parsedToken = cmd.token;
            if (!parsedToken && cmd.tokenfile) {
                parsedToken = fs.readFileSync(cmd.tokenfile, "utf-8").trim();
                cmd.token = parsedToken;
            }
            if (cmd.customerid && cmd.customerid.indexOf(JWTPrefix) == 0) {
                parsedToken = cmd.customerid;
                cmd.token = parsedToken;
            }
            if (!parsedToken) {
                throw new Error("Token Must Be Given");
            }
            var tokenData = jwtDecode(parsedToken);
            if (!tokenData["x-sl-server"]) {
                throw new Error("Token Is Invalid. Doesn't Contain Server");
            }
            if (!tokenData["subject"]) {
                throw new Error("Token Is Invalid. Doesn't Contain Subject");
            }
            var customerId = tokenData["subject"];
            var subjectParts = tokenData["subject"].split("@");
            if (subjectParts.length >= 1) {
                customerId = subjectParts[0];
            }
            cmd.server = tokenData["x-sl-server"];
            cmd.customerid = customerId;
            cmd.isValidToken = true;
        };
        Utils.prototype.createBuildSessionId = function (cmd, callback) {
            try {
                var request = {
                    appName: cmd.appname,
                    buildName: cmd.build,
                    branchName: cmd.branch,
                    buildSessionId: cmd.buildsessionid
                };
                var backendProxy = new sl_js_infra_1.SlJsInfra.BackendProxy(agent_instance_data_1.agentInstanceData, cmd, this.logger);
                backendProxy.createBuildSessionId(request, function (err, response) {
                    callback(err, response.buildSessionId);
                });
            }
            catch (err) {
                callback(err);
            }
        };
        Utils.prototype.getBuildSession = function (cmd) {
            if (!cmd.buildsessionid && !cmd.buildsessionidfile) {
                throw new Error("buildsessionid or buildsessionfile must be given");
            }
            var buildSessionId;
            if (cmd.buildsessionid) {
                buildSessionId = cmd.buildsessionid;
            }
            else {
                cmd.buildsessionidfile = cmd.buildsessionidfile || DefaultFileUtils.tryGetDefaultFile(defualtBuildSessionIdFile, cmd.workspacepath, this.logger);
                cmd.buildsessionid = fs.readFileSync(cmd.buildsessionidfile, "utf-8");
                buildSessionId = cmd.buildsessionid;
            }
            return buildSessionId;
        };
        Utils.prototype.getBuildSessionParams = function (buildSessionId, cmd, callback) {
            if (slEnvVars.inProductionListenerMode()) {
                var params = this.getSealightsProductionPrams(buildSessionId);
                return callback(null, params);
            }
            try {
                var backendProxy = new sl_js_infra_1.SlJsInfra.BackendProxy(agent_instance_data_1.agentInstanceData, cmd, this.logger);
                backendProxy.getBuildSession(buildSessionId, function (err, response) {
                    callback(err, response);
                });
            }
            catch (err) {
                callback(err);
            }
        };
        Utils.prototype.mergeParams = function (cmd, buildSessionParams) {
            //In order to merge, have all the keys in buildSessionParams as lower case
            for (var key in buildSessionParams) {
                var value = buildSessionParams[key];
                if (key == "buildName") {
                    key = "build";
                }
                else if (key == "branchName") {
                    key = "branch";
                }
                buildSessionParams[key.toLocaleLowerCase()] = value;
            }
            ["appname", "branch", "build", "customerid", "server"].forEach(function (param) {
                if (!cmd[param] && buildSessionParams[param]) {
                    cmd[param] = buildSessionParams[param];
                }
            });
        };
        Utils.prototype.validateParams = function (cmd) {
            var errors = [];
            ["customerid", "server"].forEach(function (param) {
                if (!cmd[param]) {
                    errors.push(param + " is required");
                }
            });
            if (errors.length) {
                errors.forEach(function (error) {
                    console.error(error);
                });
                throw new Error("Invalid Params Given");
            }
        };
        Utils.prototype.parseAndGetParams = function (cmd, buildSessionId, callback) {
            var _this = this;
            try {
                this.parseToken(cmd);
                this.getBuildSessionParams(buildSessionId, cmd, function (err, buildSessionParams) {
                    if (err) {
                        throw new Error("Failed Getting Build Session Params. Error: " + err);
                    }
                    _this.mergeParams(cmd, buildSessionParams);
                    _this.validateParams(cmd);
                    if (fs.existsSync(SEALIGHTS)) {
                        fs.unlink(SEALIGHTS, function () { });
                    }
                    callback(null);
                });
            }
            catch (err) {
                callback(err);
            }
        };
        Utils.prototype.validateRequiredParams = function (cmd, requiredParams, requiredLists) {
            cmd.tokenfile = cmd.tokenfile || DefaultFileUtils.tryGetDefaultFile(defualtTokenFile, cmd.workspacepath, this.logger);
            var requiredErrors = [];
            requiredParams.forEach(function (requiredParam) {
                if (!cmd[requiredParam]) {
                    requiredErrors.push(requiredParam + " is required");
                }
            });
            var atLeastOneErrors = this.validateRequiredAtLeastOneParams(cmd, requiredLists);
            var errors = requiredErrors.concat(atLeastOneErrors);
            if (errors.length) {
                var error = errors.join("\n");
                throw new Error(error);
            }
        };
        Utils.prototype.validateRequiredAtLeastOneParams = function (cmd, requiredLists) {
            var errors = [];
            requiredLists.forEach(function (requiredList) {
                var isValid = false;
                requiredList.forEach(function (param) {
                    if (cmd[param]) {
                        isValid = true;
                    }
                });
                if (!isValid) {
                    errors.push("At least one of the following params must be given. " + requiredList);
                }
            });
            return errors;
        };
        Utils.prototype.runConfigCia = function (cmd, slCiaPath, spawnRuntimeArgs) {
            var configCiaMap = {
                token: "token",
                customerid: "customerid",
                server: "server",
                proxy: "proxy",
                appname: "appname",
                excludedpaths: "nodejsexcludedpaths",
                usebranchcoverage: "usebranchcoverage"
            };
            spawnRuntimeArgs = spawnRuntimeArgs || [];
            var ciaLocation = require.resolve(slCiaPath);
            var configCiaParams = this.createParamsFromCommand(configCiaMap, cmd);
            var configCiaResult = childProcess.spawnSync(process.execPath, spawnRuntimeArgs.concat([ciaLocation + "/../sl-config-cia"]).concat(configCiaParams), {
                encoding: "utf-8",
                stdio: [0, 1, 2]
            });
            return configCiaResult.status;
        };
        Utils.prototype.runCia = function (cmd, slCiaPath, spawnRuntimeArgs) {
            var ciaMap = {
                workspacepath: "workspacepath",
                build: "build",
                branch: "branch",
                appname: "appname",
                scm: "scm",
                dependenciesFile: "dependenciesFile",
                dependency: "dependency",
                usebranchcoverage: "usebranchcoverage",
                commit: "commit",
                config: "config",
                jobName: "jobName",
                logsUrl: "logsUrl",
                buildsessionid: "buildsessionid",
                instrumentForBrowsers: "instrumentForBrowsers",
                es6Modules: "es6Modules",
                instrumentationOnly: "instrumentationOnly",
                sendContributors: "sendContributors",
                delayShutdownInSeconds: "delayShutdownInSeconds",
                uniqueModuleId: "uniqueModuleId",
                downloadAgent: "downloadAgent",
                esModules: "es6Modules",
                recursive: "recursive",
                outputpath: "outputpath",
                useNewBuildScannerProtocol: "useNewBuildScannerProtocol",
                resolveWithoutHash: "useNewBuildScannerProtocol",
                useBabylon: "useBabylon",
                scmBaseUrl: "scmBaseUrl",
                scmVersion: "scmVersion",
                scmProvider: "scmProvider",
                babylonPlugins: "babylonPlugins",
                projectRoot: "projectRoot",
                labid: "labid"
            };
            spawnRuntimeArgs = spawnRuntimeArgs || [];
            var ciaLocation = require.resolve(slCiaPath);
            var ciaParams = this.createParamsFromCommand(ciaMap, cmd);
            var ciaResult = childProcess.spawnSync(process.execPath, spawnRuntimeArgs.concat([ciaLocation + "/../sl-cia"]).concat(ciaParams), {
                encoding: "utf-8",
                stdio: [0, 1, 2]
            });
            return ciaResult.status;
        };
        Utils.prototype.saveConfigurationToProcessEnv = function (cmd) {
            var agentConfig = this.createAgentConfig(cmd);
            process.env.SL_CONFIGURATION = JSON.stringify(agentConfig.toJsonObject());
        };
        Utils.prototype.runSlNode = function (args, slNodePath, spawnRuntimeArgs) {
            spawnRuntimeArgs = spawnRuntimeArgs || [];
            var slNodeLocation = require.resolve(slNodePath);
            var slNodeParams = [slNodeLocation + "/../lib/sl-node"];
            var slNodeResult = childProcess.spawnSync(process.execPath, spawnRuntimeArgs.concat(slNodeParams).concat(args), {
                encoding: "utf-8",
                stdio: [0, 1, 2]
            });
            if (slNodeResult.status != 0) {
                throw new Error("Failed Running The Test Listener With App");
            }
        };
        Utils.prototype.runTestListener = function (cmdArgs, extraParams) {
            this.saveConfigurationToProcessEnv(cmdArgs);
            var dependencies = this.deps();
            this.runSlNode2(extraParams, dependencies.slNode2Path, this.getSpawnNodeArgs(), cmdArgs);
        };
        Utils.prototype.runSlNode2 = function (args, slNodePath, spawnRuntimeArgs, cmdArgs) {
            spawnRuntimeArgs = spawnRuntimeArgs || [];
            var useIstanbul = this.isUseIstanbul();
            var envVariables = process.env;
            envVariables["SL_httpClientColoring"] = cmdArgs.httpClientColoring;
            envVariables["SL_httpServerColoring"] = cmdArgs.httpServerColoring;
            envVariables["SL_useIstanbul"] = String(useIstanbul).toLowerCase();
            var AgentLoader = this.getAgentLoader();
            AgentLoader.run(args, useIstanbul, spawnRuntimeArgs);
        };
        Utils.prototype.getAgentLoader = function () {
            var AgentLoader = require("sl-node2/tsOutputs/lib/agent-loader").AgentLoader;
            return AgentLoader;
        };
        Utils.prototype.runMocha = function (cmdArgs, mochaParams) {
            var dependencies = this.deps();
            this.runSlNodeCoverWithMocha2(mochaParams, dependencies.slNode2Path, dependencies.slNodeMocha2Path, this.getSpawnNodeArgs(), cmdArgs);
        };
        Utils.prototype.runSlNodeCoverWithMocha2 = function (userParamsForMocha, slNode2Path, slNodeMochaPath2, spawnRuntimeArgs, cmdArgs) {
            spawnRuntimeArgs = spawnRuntimeArgs || [];
            var useIstanbul = this.isUseIstanbul();
            slNodeMochaPath2 = require.resolve(slNodeMochaPath2);
            var mochaPath = this.getMochaPath();
            var mochaParams = [require.resolve(mochaPath)];
            if (useIstanbul == true) {
                mochaParams.push("--");
            }
            mochaParams.push("-R");
            mochaParams.push(slNodeMochaPath2);
            mochaParams = mochaParams.concat(userParamsForMocha);
            this.saveConfigurationToProcessEnv(cmdArgs);
            var envVariables = process.env;
            envVariables["SL_httpClientColoring"] = cmdArgs.httpClientColoring;
            envVariables["SL_httpServerColoring"] = cmdArgs.httpServerColoring;
            var AgentLoader = this.getAgentLoader();
            AgentLoader.run(mochaParams, useIstanbul, spawnRuntimeArgs, false);
        };
        Utils.prototype.runSlNodeCoverWithJasmine = function (args, spawnRuntimeArgs) {
            var jasmineIntegrationFile = require.resolve('./integrations/jasmine/jasmine-1.x.js');
            var testDir = args[args.length - 1]; //Node-jasmine requires this to be the tests dir
            var integrationFileFullPath, jasmineIntegrationFileContents, deleteIntegrationFileAfterExecution = false;
            if (testDir) { //Can't copy integration file if no dir is specified (jasmine-node will fail anyway without this required argument)
                jasmineIntegrationFileContents = fs.readFileSync(jasmineIntegrationFile, "utf8");
                integrationFileFullPath = path.join(testDir, "_temp_sealights-spec.js"); //Important! filename must end with "-spec.js"
                deleteIntegrationFileAfterExecution = false;
                if (!fs.existsSync(integrationFileFullPath)) {
                    fs.writeFileSync(integrationFileFullPath, jasmineIntegrationFileContents);
                    deleteIntegrationFileAfterExecution = true;
                }
            }
            var jasmineNodeCommand = [
                "node_modules/jasmine-node/bin/jasmine-node",
            ];
            var jasmineParams = jasmineNodeCommand.concat(args);
            var AgentLoader = this.getAgentLoader();
            AgentLoader.run(jasmineParams, false, spawnRuntimeArgs);
            process.on("exit", function () {
                //If the integration file has been added, remove it now
                if (deleteIntegrationFileAfterExecution && fs.existsSync(integrationFileFullPath)) {
                    fs.unlinkSync(integrationFileFullPath);
                }
            });
        };
        Utils.prototype.configTestFramework = function (cmd, callback) {
            var _this = this;
            try {
                this.validateRequiredParams(cmd, [], [
                    ["token", "tokenfile"],
                    ["buildsessionid", "buildsessionidfile"]
                ]);
                var buildSessionId = this.getBuildSession(cmd);
                this.parseAndGetParams(cmd, buildSessionId, function (err) {
                    if (err) {
                        callback(err);
                    }
                    else {
                        if (!cmd.labid) {
                            cmd.labid = buildSessionId;
                        }
                        _this.saveConfigurationToProcessEnv(cmd);
                        callback(null);
                    }
                });
            }
            catch (err) {
                console.error(err.message);
                var statusCode = cmd.failbuild ? 1 : 0;
                var error = new Error("status code: " + statusCode);
                error.statusCode = statusCode;
                callback(error);
            }
        };
        Utils.prototype.getSpawnNodeArgs = function () {
            var _this = this;
            var args = [];
            process.execArgv.forEach(function (arg) {
                if (_this.isAllowedNodeArg(arg)) {
                    args.push(arg);
                }
            });
            return args;
        };
        Utils.prototype.isAllowedNodeArg = function (nodeArg) {
            var optionName = this.getNodeArgName(nodeArg);
            return allowedV8Options.indexOf(optionName) > -1;
        };
        Utils.prototype.getNodeArgName = function (nodeArg) {
            nodeArg = nodeArg.replace("--", "");
            var equalIndex = nodeArg.indexOf("=");
            if (equalIndex > -1) {
                nodeArg = nodeArg.substring(0, equalIndex);
            }
            return nodeArg.replace(/-/g, "_");
        };
        Utils.prototype.handleError = function (cmd, err) {
            console.error(err.message);
            if (cmd.failbuild) {
                err.statusCode = 1;
                return err;
            }
            return null;
        };
        Utils.prototype.getMochaPath = function () {
            var mochaPath = /^win/.test(process.platform) ? "mocha/bin/_mocha" : ".bin/_mocha";
            return mochaPath;
        };
        Utils.prototype.isUseIstanbul = function () {
            var useIstanbul = process.env["SL_useIstanbul"] != null ? process.env["SL_useIstanbul"] : false;
            if (useIstanbul != null) {
                useIstanbul = (String(useIstanbul).toLowerCase() === 'true');
            }
            return useIstanbul;
        };
        ;
        Utils.prototype.createAgentConfig = function (cmd) {
            var agentConfig = new sl_js_infra_1.SlJsInfra.AgentConfig();
            var agentConfigKeys = agentConfig.getLowerCaseToKeyMap();
            Object.keys(cmd).forEach(function (key) {
                var property = agentConfigKeys[key.toLowerCase()];
                if (property) {
                    agentConfig[property].value = cmd[key];
                }
            });
            return agentConfig;
        };
        Utils.prototype.getSealightsProductionPrams = function (buildSessionId) {
            return {
                appName: "Sealights-prod",
                branchName: "prod/master",
                buildSessionId: buildSessionId,
                buildName: "Sealights-prod_1",
            };
        };
        return Utils;
    }());
    exports.Utils = Utils;
});
//# sourceMappingURL=utils.js.map