(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "url"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var URL = require("url");
    var UrlBuilder = /** @class */ (function () {
        function UrlBuilder() {
            this.path = '';
            this.paramsMap = new Map();
        }
        UrlBuilder.prototype.withHost = function (host) {
            this.host = this.resolveUrl(host);
            if (this.host.endsWith("/")) {
                this.host = this.host.substring(0, this.host.length - 1);
            }
            return this;
        };
        UrlBuilder.prototype.withPath = function () {
            var pathParams = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                pathParams[_i] = arguments[_i];
            }
            for (var i = 0; i < pathParams.length; i++) {
                this.path += encodeURIComponent(pathParams[i]) + "/";
            }
            return this;
        };
        UrlBuilder.prototype.withQueryParam = function (key, value) {
            this.paramsMap.set(encodeURIComponent(key), encodeURIComponent(value));
            return this;
        };
        UrlBuilder.prototype.toString = function () {
            var finalUrl = this.host + "/" + this.path;
            if (this.paramsMap.size == 0) {
                return finalUrl;
            }
            return finalUrl + this.buildQueryParams();
        };
        UrlBuilder.prototype.resolveUrl = function (url) {
            if (!(url.startsWith("http://") || url.startsWith("https://"))) {
                url = "http://" + url;
            }
            return URL.parse(url).href;
        };
        UrlBuilder.prototype.buildQueryParams = function () {
            var _this = this;
            var queryString = '';
            this.paramsMap.forEach(function (value, key) {
                queryString += _this.addQueryStringValue(key, value);
            });
            queryString = "?" + queryString;
            queryString = queryString.substring(0, queryString.length - 1); //Remove the last &.
            return queryString;
        };
        UrlBuilder.prototype.addQueryStringValue = function (key, value) {
            if (!key && !value)
                return '';
            var paramString = '';
            paramString += key;
            paramString += "=";
            paramString += value;
            paramString += "&";
            return paramString;
        };
        return UrlBuilder;
    }());
    exports.UrlBuilder = UrlBuilder;
});
//# sourceMappingURL=url-builder.js.map